#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Tokens using a Sliding Window Algorithm """


from pprint import pformat

from baseblock import Stopwatch
from baseblock import BaseObject


class SlidingWindowExtract(BaseObject):
    """ Extract Tokens using a Sliding Window Algorithm """

    def __init__(self,
                 tokens: list,
                 gram_size: int):
        """
        Created:
            24-Aug-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)
        self._tokens = tokens
        self._gram_size = gram_size

    @staticmethod
    def _normalize(results: list) -> list:
        normalized = []
        for result in results:

            output = result
            if type(result) == list:
                output = ' '.join(result).strip()

            normalized.append(output.replace('  ', ' ').strip())

        return normalized

    def _process(self) -> list:
        results = []
        tokens = self._tokens

        if self._gram_size == len(tokens):
            return self._normalize(tokens)

        if self._gram_size == 1:
            return self._normalize(tokens)

        x = 0
        y = x + self._gram_size

        while y <= len(tokens):
            results.append(tokens[x: y])

            x += 1
            y = x + self._gram_size

        normalized = []
        for result in results:
            output = ' '.join(result).strip()
            output = output.replace('  ', ' ')
            normalized.append(output)

        return normalized

    def process(self) -> list:
        sw = Stopwatch()

        results = self._process()

        if self.isEnabledForDebug:

            self.logger.debug('\n'.join([
                "Sliding Window Extract Completed",
                f"\tGram Size: {self._gram_size}",
                f"\tTotal Tokens: {len(results)}",
                f"\tTotal Time: {str(sw)}"]))

            if self._tokens != results:
                self.logger.debug('\n'.join([
                    "Sliding Window Extract Results",
                    f"\tTokens: {pformat(results, indent=4)}"]))

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Perform Fuzzy Matching on unmatched tokens """


from typing import Callable

from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_tag.dmo import CandidateFuzzyMatch


class FuzzyMatchFinder(BaseObject):
    """ Perform Fuzzy Matching on Tokens that don't have an exact match to the dbpedia tries """

    def __init__(self,
                 entity_finder: Callable):
        """
        Created:
            24-Aug-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)
        self._fuzzy_match = CandidateFuzzyMatch(entity_finder).process

    def _process(self,
                 tokens: list) -> dict:

        d_fuzzy = {}

        for token in tokens:
            if type(token) == list:
                token = ' '.join(token)
            d_fuzzy[token] = self._fuzzy_match(token)

        return d_fuzzy

    def process(self,
                tokens: list) -> list:
        sw = Stopwatch()

        d_fuzzy = self._process(tokens)

        if self.isEnabledForInfo:
            if self.isEnabledForInfo:
                self.logger.info('\n'.join([
                    "Sliding Window Completed",
                    f"\tTotal Results: {len(d_fuzzy)}",
                    f"\tTotal Time: {str(sw)}"]))

        return d_fuzzy



from .bp.tagger import Tagger
from .bp import *
from .svc import *
from .dmo import *
from .dto import *


import logging


components = [
    'dbpedia_tag.dmo.candidate_fuzzy_match',
    'dbpedia_tag.dmo.sliding_window_blacklist',
    'dbpedia_tag.dmo.sliding_window_validate',
]

[logging.getLogger(x).setLevel(logging.INFO) for x in components]


tagapi = Tagger()


def dbpedia_tag(input_text) -> dict:
    return tagapi.process(input_text)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from pprint import pprint

from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject
from baseblock import ServiceEventGenerator

from fast_sentence_tokenize import Tokenizer

from dbpedia_ent.bp import Finder
from dbpedia_tag.svc import ExactMatchFinder
from dbpedia_tag.svc import FuzzyMatchFinder


class Tagger(BaseObject):
    """ Orchestrate Taxonomy Generation """

    def __init__(self):
        """ Change Log:

        Created:
            24-Aug-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)
        finder = Finder()
        self._entity_exists = finder.exists
        self._tokenize = Tokenizer().input_text

        self._generate_event = ServiceEventGenerator().process
        self._fuzzy_matcher = FuzzyMatchFinder(finder.find).process

    def _tag_exact(self,
                   d_coords: dict,
                   d_canons: dict,
                   input_text: str,
                   original_text: str):

        for i in reversed(range(1, 4)):

            tokens = [x.strip() for x in self._tokenize(input_text)]
            tokens = [x for x in tokens if not x.startswith('entity_')]

            svc = ExactMatchFinder(gram_size=i,
                                   entity_exists=self._entity_exists)

            d_finder = svc.process(tokens)
            if not d_finder:
                continue

            if d_finder['validated']:
                for item in d_finder['validated']:

                    key = f"entity_{item.replace(' ', '_').lower()}"

                    x = original_text.lower().index(item)
                    y = x + len(item)

                    d_coords[key] = [x, y]
                    d_canons[key] = item

                    input_text = input_text.replace(item, key)
                    print(input_text)
                    print('\n'*2)

                tokens = [x.strip() for x in self._tokenize(input_text)]
                tokens = [x for x in tokens if not x.startswith('entity_')]

            if d_finder['unmatched']:

                d_fuzzy = self._fuzzy_matcher(d_finder['unmatched'])
                d_fuzzy = {k: d_fuzzy[k] for k in d_fuzzy if d_fuzzy[k]}

                for k in d_fuzzy:
                    if not d_fuzzy[k]:
                        continue

                    key = f"Fuzzentity_{d_fuzzy[k].replace(' ', '_').lower()}"

                    x = original_text.lower().index(k)
                    y = x + len(k)

                    d_coords[key] = [x, y]
                    d_canons[key] = d_fuzzy[k]

                    input_text = input_text.replace(k, key)

        return input_text

    def process(self,
                input_text: str):

        sw = Stopwatch()
        output_events = []

        d_canons = {}
        d_coords = {}

        original_text = input_text
        input_text = input_text.lower().strip()

        input_text = self._tag_exact(
            d_coords=d_coords,
            d_canons=d_canons,
            input_text=input_text,
            original_text=original_text)

        # input_text = self._tag(d_coords=d_coords,
        #                        d_canons=d_canons,
        #                        input_text=input_text,
        #                        original_text=original_text,
        #                        tag_bucket='validated')

        output_events.append(self._generate_event(
            service_name=self.component_name(),
            event_name='tagger',
            stopwatch=sw,
            data={
                'input_text': original_text,
                'output_text': input_text,
                'canons': d_canons,
                'coords': d_coords,
            }))

        return {
            'text': input_text,
            'events': output_events
        }




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

## ---------------------------------------------------------- ##
# Purpose:    List of Stopwords
# Notes:      this list will likely be refined as needed over time
#             the 'stopwords' represented here are likely too coarse-grained
## ---------------------------------------------------------- ##
stopwords = [
    'a', 'about', 'above', 'after', 'again', 'against', 'all', 'am', 'an', 'and', 'any', 'are', 'as', 'at', 'be', 'because', 'been', 'before',
    'between', 'both', 'but', 'by', 'can', 'did', 'do', 'does', 'doing', 'don', 'down', 'during', 'each', 'few', 'for', 'from', 'further',
    'had', 'has', 'have', 'having', 'he', 'her', 'here', 'hers', 'herself', 'him', 'himself', 'his', 'how', 'i', 'if', 'in', 'into', 'is', 'it', 'its', 'itself',
    'just', 'me', 'more', 'most', 'my', 'myself', 'no', 'nor', 'not', 'now', 'of', 'off', 'on', 'once', 'only', 'or', 'other', 'our', 'ours', 'ourselves', 'out',
    'over', 'own', 's', 'same', 'she', 'should', 'so', 'some', 'such', 't', 'than', 'that', 'the', 'their', 'theirs', 'them', 'themselves', 'then', 'there',
    'these', 'they', 'this', 'those', 'through', 'to', 'too', 'under', 'until', 'up', 'very', 'was', 'we', 'were', 'what', 'when', 'where', 'which', 'while',
    'who', 'whom', 'why', 'will', 'with', 'you', 'your', 'yours', 'yourself', 'yourselves',

    # entities that are recognized as conveying no value ...
    'and i',
    'i am',
    'we can',
    'you are',
    'want to',
]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Validate Tokens against the dbPedia Dictionaries """


from pprint import pformat
from typing import Callable
from wsgiref import validate

from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_ent import Finder


class SlidingWindowValidate(BaseObject):
    """ Validate Tokens against the dbPedia Dictionaries """

    def __init__(self,
                 candidates: list,
                 entity_exists: Callable):
        """
        Created:
            24-Aug-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)
        self._candidates = candidates
        self._entity_exists = entity_exists

    def _process(self) -> list:

        normalized = []

        for candidate in self._candidates:

            def get_entity() -> str:
                if type(candidate) == list:
                    return ' '.join(candidate)
                return candidate

            entity = get_entity()
            if len(entity) < 3:
                continue

            if self._entity_exists(entity):
                normalized.append(entity)

        return normalized

    def process(self) -> list:
        sw = Stopwatch()

        results = self._process()

        if self.isEnabledForDebug:

            self.logger.debug('\n'.join([
                "Sliding Window Validation Completed",
                f"\tTotal Tokens: {len(results)}",
                f"\tTotal Time: {str(sw)}"]))

            if self._candidates != results:
                self.logger.debug('\n'.join([
                    "Sliding Window Validation Results",
                    f"\tTokens: {pformat(results, indent=4)}"]))

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Blacklist certain Tokens from the sliding window result """


from pprint import pformat

from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_tag.dto import stopwords


class SlidingWindowBlacklist(BaseObject):
    """ Blacklist certain Tokens from the sliding window result """

    def __init__(self):
        """
        Created:
            24-Aug-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def _process(self,
                 candidates: list) -> list:
        for candidate in candidates:
            print (candidate)
            if type(candidate) == list:
                raise ValueError
        return [x for x in candidates if x not in stopwords]

    def process(self,
                candidates: list) -> list:
        sw = Stopwatch()

        results = self._process(candidates)

        if self.isEnabledForDebug:

            self.logger.debug('\n'.join([
                "Sliding Window Blacklist Completed",
                f"\tTotal Tokens: {len(candidates)}",
                f"\tTotal Time: {str(sw)}"]))

            if candidates != results:
                self.logger.debug('\n'.join([
                    "Sliding Window Blacklist Completed",
                    f"\tTokens: {pformat(candidates, indent=4)}"]))

        return candidates
